import os
from gooey.python_bindings.gooey_decorator import Gooey
from gooey.python_bindings.gooey_parser import GooeyParser
# from gooey.gui import application

version_file = os.path.join(os.path.dirname(__file__), 'version')

__version__ = '0.9.2.5'
